(* ::Package:: *)

Needs["xAct`xTensor`"];
$PrePrint = ScreenDollarIndices

(********** 4-manifold ************)
DefManifold[M4,4,{a,b,c,d,e,f,g,h,i,j,k,l,m}];

DefConstantSymbol@{c1,c2,c3,c4};

(********** metric **********)
DefMetric[-1, metricg[-a, -b], CD, {";", "\[Del]"}, PrintAs -> "g"];

(***** timelike normal to the slice *********)
DefTensor[n[-a], {M4}, PrintAs->"n"];
AutomaticRules[n,MakeRule[{n[a]n[-a],-1}, MetricOn -> All]];
AutomaticRules[n,MakeRule[{metricg[-a,-b]n[a]n[b],-1}, MetricOn -> All]];
AutomaticRules[n, MakeRule[{n[a] CD[b][n[-a]], 0}, MetricOn -> All]]

(***** Define induced metric ******)
DefMetric[1,metricgn[-a,-b],cd,SymbolOfCovD->{"|","D"},InducedFrom->{metricg,n},PrintAs->"\[Gamma]"];

(***** Sign conventions *********)
$ExtrinsicKSign=-1;
$AccelerationSign=-1;

(***** Define lapse  ******)
DefTensor[Lapsen[], {M4}, PrintAs -> "\[Alpha]"];
LapseRule:=MakeRule[{Accelerationn[-a],cd[-a][Lapsen[]]/Lapsen[]},MetricOn->All];

(***** Lorentz factor, boost vector and four-velocity vector ****)
DefTensor[W[], {M4}];
DefTensor[v[-a],M4,OrthogonalTo->{n[a]},ProjectedWith->{metricgn[a,-b]}];
DefTensor[vhat[-a],M4,OrthogonalTo->{n[a]},ProjectedWith->{metricgn[a,-b]}];
DefTensor[u[-a], M4,PrintAs->"u"];
AutomaticRules[u, MakeRule[{u[a] n[-a], -W[]}, MetricOn -> All]];
AutomaticRules[u, MakeRule[{metricg[-a,-b] n[a] u[b], -W[]}, MetricOn->All]];
AutomaticRules[u, MakeRule[{metricgn[-a,-b]u[b], vhat[-a]}, MetricOn->All]];
AutomaticRules[u, MakeRule[{u[a] u[-a], -1}, MetricOn -> All]];
AutomaticRules[u, MakeRule[{u[a] CD[b][u[-a]], 0}, MetricOn -> All]];


(*-----------------------------------------------------------------------------------------------------------------------------------*)

(***** Define DF-properties and Rules  ******)
DefTensor[V[-a],M4,OrthogonalTo->{u[a]}];
DefTensor[metricgu[-a,-b], M4, Symmetric[{1, 2}], OrthogonalTo->{u[a]}, PrintAs -> "\[CapitalUpsilon]"];

metricgutoguRule:=MakeRule[{metricgu[-a,-b],metricg[-a,-b]+u[-a] u[-b]},ContractMetrics -> True];

gtoUpsilonRule:=MakeRule[{metricg[-a,-b],metricgu[-a,-b]-u[-a] u[-b]},MetricOn->All, ContractMetrics -> True];
UpsilontogRule:=MakeRule[{metricgu[-a,-b],metricg[-a,-b]+u[-a] u[-b]},MetricOn->All, ContractMetrics -> True];
utoVRule:=MakeRule[{u[-a],W[]^(-1) n[-a]-V[-a]}, MetricOn->All, ContractMetrics -> True];
utovhatRule:=MakeRule[{u[-a],W[]n[-a]+vhat[-a]}, MetricOn->All, ContractMetrics -> True];
CDutoCDvhatRule:=MakeRule[{CD[-b][u[-a]],CD[-b][W[]n[-a]+vhat[-a]]}, MetricOn->All, ContractMetrics -> True];

WtoVRule:=MakeRule[{W[],1/Sqrt[1-V[b]V[-b]]}];
vhattoVRuleDown:=MakeRule[{vhat[-a],-V[-a]+n[-a](1/W[]-W[])},MetricOn->None];
vhattoVRuleUp:=MakeRule[{vhat[a],-V[a]+n[a](1/W[]-W[])},MetricOn->None];
VtovhatRule:=MakeRule[{V[-a],-vhat[-a]+n[-a](1/W[]-W[])},MetricOn->All, ContractMetrics -> True];
nVtoWRule := MakeRule[{n[-b] V[b], W[] - 1/W[]}, MetricOn -> All];


uunitVRule:=MakeRule[{V[-a]V[a],1-1/W[]^2}];

ntouRule :=  MakeRule[{n[a], u[a]/W[] - vhat[a]/W[]}, MetricOn -> All, ContractMetrics -> True];
gunRule :=  MakeRule[{metricgu[a, b] n[-a], V[b] W[]}, MetricOn -> All, ContractMetrics -> True];
nCDvhattoVCDnRule :=  MakeRule[{n[a] CD[-b][vhat[-a]], -CD[-b][n[a]] (-V[-a] + n[-a]*(W[]^(-1) - W[]))}, MetricOn -> All, ContractMetrics -> True];
(*-----------------------------------------------------------------------------------------------------------------------------------*)

(***** Define n-boost metric \[DoubleStruckG] ******)
DefTensor[metricgnboost[-a, -b], M4, Symmetric[{1, 2}], PrintAs -> "\[DoubleStruckG]", OrthogonalTo -> {n[a]}, ProjectedWith -> {metricgn[a, -c]}];
DefTensor[metricgnboostinv[a, b], M4, Symmetric[{1, 2}], PrintAs -> "(\!\(\*SuperscriptBox[\(\[DoubleStruckG]\), \(-1\)]\))",  OrthogonalTo -> n[-a], ProjectedWith -> {metricgn[-a, c]}]; 

(***** Several helpful boost metric Rules ******)
unitBoostRule:=MakeRule[{metricgnboost[-a, -b] metricgnboostinv[a, c], metricgn[-b, c]}, MetricOn -> All, ContractMetrics -> True];
boosttogammaRule:=MakeRule[{metricgnboost[-a, -b], metricgn[-a,-b]+vhat[-a] vhat[-b]}, MetricOn -> All, ContractMetrics -> True];
boostinvtogammaRule:=MakeRule[{metricgnboostinv[-a, -b], metricgn[-a,-b]-vhat[-a] vhat[-b]/W[]^2}, MetricOn -> All, ContractMetrics -> True];
gammatoboostRule:=MakeRule[{metricgn[-a, -b], metricgnboost[-a,-b]-vhat[-a] vhat[-b]}, MetricOn -> All, ContractMetrics -> True];
gammatoboostinvRule:=MakeRule[{metricgn[-a, -b], metricgnboostinv[-a,-b]+vhat[-a] vhat[-b]/W[]^2}, MetricOn -> All, ContractMetrics -> True];
gammatoboostinvRuleUp:=MakeRule[{metricgn[a, b], metricgnboostinv[a,b]+vhat[a] vhat[b]/W[]^2}, MetricOn -> None];

boostvhatRule:=MakeRule[{metricgnboost[-a, -b] vhat[b], W[]^2 vhat[-a]}, MetricOn -> All, ContractMetrics -> True]; 
boostinvvhatRule:=MakeRule[{metricgnboostinv[-a, -b] vhat[b], W[]^(-2) vhat[-a]}, MetricOn -> All, ContractMetrics -> True]; 

DefTensor[gnboost[-a, -b], M4]; 
gnboost[a_, b_]= metricgn[a, b] + vhat[a] vhat[b];
DefTensor[gnboostinv[-a, -b], M4]; 
gnboostinv[a_, b_] = metricgn[a, b] - 1/W[]^2 vhat[a] vhat[b];

(*-----------------------------------------------------------------------------------------------------------------------------------*)


(***** Define u-boost metric \u[DoubleStruckG] ******)
DefTensor[metricguboost[-a, -b], M4, Symmetric[{1, 2}], PrintAs -> "\!\(\*SuperscriptBox[\((\), \(u\)]\)\[DoubleStruckG])", 
  OrthogonalTo -> {u[a]}];
  
DefTensor[metricguboostinv[a, b], M4, Symmetric[{1, 2}], PrintAs -> "\!\(\*SuperscriptBox[\((\), \(u\)]\)\!\(\*SuperscriptBox[\(\
\[DoubleStruckG]\), \(-1\)]\))", OrthogonalTo -> u[-a]];

(***** Several helpful u-boost metric Rules ******)
unituBoostRule:=MakeRule[{metricguboost[-a, -b] metricguboostinv[a, c], metricgu[-b, c]}, MetricOn -> All, ContractMetrics -> True];
uboosttoguRule:=MakeRule[{metricguboost[-a, -b], metricgu[-a,-b]+W[] V[-a] W[] V[-b]}, MetricOn -> All, ContractMetrics -> True];
uboostinvtoguRule:=MakeRule[{metricguboostinv[-a, -b], metricgu[-a,-b]-V[-a] V[-b]}, MetricOn -> All, ContractMetrics -> True];
gutouboostRule:=MakeRule[{metricgu[-a, -b], metricguboost[-a,-b]-W[] V[-a] W[] V[-b]}, MetricOn -> All, ContractMetrics -> True];
gutouboostinvRule:=MakeRule[{metricgu[-a, -b], metricguboostinv[-a,-b]+V[-a] V[-b]}, MetricOn -> All, ContractMetrics -> True];
gutouboostinvRuleUp:=MakeRule[{metricgu[a, b], metricguboostinv[a,b]+V[a] V[b]}, MetricOn -> None];


ProjecteduboostinvCDvhatRule :=  MakeRule[{metricguboostinv[a, c]CD[-a][vhat[-c]], metricguboostinv[a, b] metricgn[-b, c] CD[-a][vhat[-c]] - n[c] V[a]/W[] CD[-a][vhat[-c]]},MetricOn -> All, ContractMetrics-> True];
ProjecteduboostinvCDvhatfreeIndexRule :=  MakeRule[{metricguboostinv[a, c]CD[-d][vhat[-c]], metricguboostinv[a, b] metricgn[-b, c] CD[-d][vhat[-c]] - n[c] V[a]/W[] CD[-d][vhat[-c]]},MetricOn -> All, ContractMetrics-> True];
ProjecteduboostinvCDnRule :=  MakeRule[{metricguboostinv[a, c]CD[-a][n[-c]], metricguboostinv[a, b] metricgn[-b, c] CD[-a][n[-c]] - n[c] V[a]/W[] CD[-a][n[-c]]},MetricOn -> All, ContractMetrics-> True];
ProjecteduboostinvCDnfreeIndexRule :=  MakeRule[{metricguboostinv[a, c]CD[-d][n[-c]], metricguboostinv[a, b] metricgn[-b, c] CD[-d][n[-c]] - n[c] V[a]/W[] CD[-d][n[-c]]},MetricOn -> All, ContractMetrics-> True];



SwitchboostRule := MakeRule[{metricgn[a, -b] metricguboostinv[b, -c], metricgnboostinv[a, -b] metricgu[b, -c]}, MetricOn -> All,ContractMetrics -> True];


(*-----------------------------------------------------------------------------------------------------------------------------------*)


(***** Several helpful rules for the u-relationship *********)
utonRule:=MakeRule[{u[-a],W[](n[-a]+v[-a])}, MetricOn->All];
uunitRule:=MakeRule[{v[-a]v[a],1-1/W[]^2}];
uunitVhatRule:=MakeRule[{vhat[-a]vhat[a],W[]^2-1}];
LieWnCovRule1:=MakeRule[{n[d]CD[-d][W[]],LieD[n[d]][W[]]}];
LievnCovRule:=MakeRule[{n[d]CD[-d][v[-a]],LieD[n[d]][v[-a]]-v[-d]CD[-a][n[d]]}];
vhatRule:=MakeRule[{v[-a],vhat[-a]/W[]}];
vhatRuleUp:=MakeRule[{v[a],vhat[a]/W[]}];
uDivergenceRule:=MakeRule[{CD[-a][u[a]], 1/Detmetricg[]*PD[-a][Detmetricg[]*u[a]]},MetricOn->All];
vhattovRule:=MakeRule[{vhat[-a], v[-a] W[]}];
vhattovRuleUp:=MakeRule[{vhat[a], v[a] W[]}];

WtovhatRule:=MakeRule[{W[], Sqrt[1 + Scalar[vhat[a] vhat[-a]]]}, MetricOn -> All];


CDWtovhatRule:=MakeRule[{CD[-a][W[]],CD[-a][Sqrt[1 + vhat[-a]  vhat[a]]]},MetricOn->All];
cdWtovhatRule:=MakeRule[{cd[-a][W[]],cd[-a][Sqrt[1 + vhat[-a]  vhat[a]]]},MetricOn->All];


(***** Rules for Transformation from vhat to v *****)
CDWtovRule:=MakeRule[{CD[-a][W[]],v[b]*W[]^3*CD[-a][v[-b]]},MetricOn->All,ContractMetrics->True];
CDWtovRule:=MakeRule[{CD[-a][W[]],v[b]*W[]^3*CD[-a][v[-b]]},MetricOn->All,ContractMetrics->True];
WtovRule:=MakeRule[{W[], 1/Sqrt[1 - Scalar[v[a] v[-a]]]}, MetricOn -> All];
uBiGammatonBiUpsilonRule:= MakeRule[{metricguboostinv[-i, c] metricgn[-c, b], metricgnboostinv[b, a]*metricgu[-i, -a]}, MetricOn -> All,  ContractMetrics -> True, UseSymmetries -> True];
CDvhattoCDvRule := MakeRule[{CD[-a][vhat[-b]],W[]*(CD[-a][v[-b]] + v[-b]*v[c]*W[]^2*CD[-a][v[-c]])}, MetricOn -> All, ContractMetrics -> True];
gammaCDvhattoboostCDvRule := MakeRule[{metricgn[-c, a] CD[-b][vhat[-a]], metricgnboost[-c, a]*W[]*CD[-b][v[-a]]}, MetricOn -> All,  ContractMetrics -> True];
boostinvCDvhattogammaCDvRule := MakeRule[{metricgnboostinv[-c, a] CD[-b][vhat[-a]], metricgn[-c, a]*W[]*CD[-b][v[-a]]}, MetricOn -> All,  ContractMetrics -> True];

(***** Several helpful rules for the metric-relationship *********)
gtogammadownRule:=MakeRule[{metricg[-a,-b],metricgn[-a,-b]-n[-a]n[-b]}, MetricOn->All];
gtogammaupRule:=MakeRule[{metricg[a,b],metricgn[a,b]-n[a]n[b]}, MetricOn->All];
gammatogRule:=MakeRule[{metricgn[-a,-b],metricg[-a,-b]+n[-a]n[-b]}, MetricOn->All];

(***** Define determinants *********)
AutomaticRules[Detmetricg, MakeRule[{Detmetricg[],Detmetricgn[]*Lapsen[]}]];

(***** Define time vector and Lie rule to get time derivative *********)
DefTensor[t[-a], M4];
DefTensor[\[Beta][-a], M4, OrthogonalTo->{n[a]},ProjectedWith->{metricgn[a,-c]}];
vhatToTimeRule:=MakeRule[{LieD[n[b]][vhat[-a]],1/Lapsen[]LieD[t[b] - \[Beta][b]][vhat[-a]]}];
LieWToLievhatRule:=MakeRule[{LieD[n[a]][W[]],LieD[n[a]][Sqrt[1 + vhat[-a] metricgn[a, b] vhat[-b]]]}];
RhoToTimeRule:=MakeRule[{LieD[n[b]][\[Rho]0[]],1/Lapsen[]LieD[t[b] - \[Beta][b]][\[Rho]0[]]}];

(***** pressure relations an speed of sound *********)
DefConstantSymbol[\[Chi]];
DefConstantSymbol[\[Kappa]];
DefConstantSymbol[Cs];
PDpToMatterRule:=MakeRule[{PD[-a][p[]],\[Chi]PD[-a][\[Rho]0[]]+\[Kappa]PD[-a][\[Epsilon][]]},MetricOn -> All];
CovDpToMatterRule:=MakeRule[{CD[-a][p[]],\[Chi] CD[-a][\[Rho]0[]]+\[Kappa] CD[-a][\[Epsilon][]]},MetricOn -> All];
KappapToSpeedofSoundRule:=MakeRule[{\[Kappa] p[], Cs^2*\[Rho]0[]^2*(enth[]) -\[Chi] \[Rho]0[]^2}];
EliminateRhoRule :=  MakeRule[{CD[-a][\[Rho]0[]],  1/\[Chi] (CD[-a][p[]] - \[Kappa] CD[-a][\[Epsilon][]])},  MetricOn -> All, ContractMetrics -> True];
epsrhotoenthRule =  MakeRule[{\[Epsilon][] \[Rho]0[],  enth[] \[Rho]0[] - \[Rho]0[] - p[]}];

(***** Rules for special problems *********)
ToDownCovvhat:=MakeRule[{metricgn[-a, -b] cd[b][vhat[a]], metricgn[a, b] cd[-b][vhat[-a]]}, MetricOn -> All];