(* ::Additional Package FOR MHD:: *)

Needs["xAct`xTensor`"];
$PrePrint = ScreenDollarIndices

(********* Magnetic field *********)
DefTensor[bm[-a], M4, PrintAs -> "b", OrthogonalTo -> {u[a]}];
DefTensor[bhat[-a], M4, OrthogonalTo -> {n[a]}, ProjectedWith -> {metricgn[a, -b]}];
DefTensor[Bm[-a], M4, PrintAs -> "B", OrthogonalTo -> {n[a]}, ProjectedWith -> {metricgn[a, -b]}];

DefTensor[Normbm[], M4,  PrintAs -> "\!\(\*SuperscriptBox[\(b\), \(2\)]\)"];
DefTensor[NormBm[], M4, PrintAs -> "\!\(\*SuperscriptBox[\(B\), \(2\)]\)"];

(********* Rules for the Magnetic fields Relationship *********)
bmtonRule := MakeRule[{bm[-a], vhat[b]*Bm[-b] n[-a] + Bm[-a]/W[] + Bm[c] vhat[-c]/W[]*vhat[-a]}, MetricOn -> All];
bmtobhatRule:=MakeRule[{bm[-a], vhat[b]*bhat[-b]/W[] n[-a] + bhat[-a]},  MetricOn -> All];
bhattoBmRule:= MakeRule[{bhat[-a], Bm[-c]/W[] metricgnboost[c, -a]}, MetricOn -> All, ContractMetrics -> True];
BmtobhatRule:=MakeRule[{Bm[-a],W[] bhat[-c] metricgnboostinv[c,-a]},MetricOn -> All, ContractMetrics -> True];

ToNormBmRule:=MakeRule[{Bm[a]Bm[-a], NormBm[]}]; 
ToNormbmRule:=MakeRule[{bm[a] bm[-a], Normbm[]}];
NormBmtoBmRule:=MakeRule[{NormBm[], Bm[a]Bm[-a]}];
NormbmTobmRule:=MakeRule[{Normbm[], bm[a]bm[-a]}];
NormbmtoBmboostRule:=MakeRule[{Normbm[], Bm[a] Bm[b] metricgnboost[-a, -b]/W[]^2}];
NormbmtobhatboostinvRule:=MakeRule[{Normbm[], bhat[a] bhat[b] metricgnboostinv[-a, -b]}];



(********* Electric field *********)
DefTensor[em[-a], M4, PrintAs -> "e", OrthogonalTo -> {u[a]},ProjectedWith -> {metricgu[a, -b]}];
DefTensor[ehat[-a], M4, OrthogonalTo -> {n[a]}, ProjectedWith -> {metricgn[a, -b]}];
DefTensor[Em[-a], M4, PrintAs -> "E", OrthogonalTo -> {n[a]},  ProjectedWith -> {metricgn[a, -b]}];

DefTensor[Normem[], M4,  PrintAs -> "\!\(\*SuperscriptBox[\(e\), \(2\)]\)"];
DefTensor[NormEm[], M4, PrintAs -> "\!\(\*SuperscriptBox[\(E\), \(2\)]\)"];

(********* Rules for the Electric fields Relationship *********)
emtoehatRule:=MakeRule[{em[-a], vhat[b]*ehat[-b]/W[] n[-a] + ehat[-a]},  MetricOn -> All];
CDemtoehatRule:=MakeRule[{CD[-c][em[-a]],CD[-c][vhat[b]*ehat[-b]/W[] n[-a] + ehat[-a]]}, MetricOn -> All];
ehattoemRuleUp :=  MakeRule[{ehat[a], em[a] - n[a] ehat[b] vhat[-b]/W[]}, MetricOn -> None];

ToNormEmRule:=MakeRule[{Em[a]Em[-a], NormEm[]}]; 
ToNormemRule:=MakeRule[{em[a] em[-a], Normem[]}];
NormEmtoEmRule:=MakeRule[{NormEm[], Em[a]Em[-a]}];
NormemToemRule:=MakeRule[{Normem[], em[a]em[-a]}];
NormemtoehatboostinvRule:=MakeRule[{Normem[], ehat[a] ehat[b] metricgnboostinv[-a, -b]}];


(********* Define New Pressure and Enthalpy *********)
DefTensor[pm[], M4, PrintAs -> "\!\(\*SuperscriptBox[\(p\), \(*\)]\)"];
DefTensor[enthm[], M4, PrintAs -> "\!\(\*SuperscriptBox[\(h\), \(*\)]\)"];
DefTensor[enthM[], M4, PrintAs -> "Hm"];

(********* Rules for the old to new Pressure and Enthalpie and vice versa *********)
InsertHmRule:= MakeRule[{enth[],1/(\[Rho]0[] W[]^2)(enthM[]-NormBm[])}];
InserthmRule:=MakeRule[{\[Epsilon][] \[Rho]0[], \[Rho]0[] enthm[] - \[Rho]0[] - p[] - bm[a] bm[-a]}];
enthmtoenthRule:= MakeRule[{enthm[], enth[] + Normbm[]/\[Rho]0[]}];
HmtohRule:=MakeRule[{enthM[],enth[] \[Rho]0[] W[]^2 + NormBm[] }];
enthtopRule:=MakeRule[{enth[],1+\[Epsilon][]+p[]/( \[Rho]0[] )}];

InsertpmRule:= MakeRule[{p[], pm[] - Normbm[]/2}];
InsertCovDpmRule:=MakeRule[{\[Chi] CD[-d][\[Rho]0[]], CD[-d][pm[]]- bm[a] CD[-d][bm[-a]] - \[Kappa] CD[-d][\[Epsilon][]]}];
pmtopRule:=MakeRule[{pm[],p[]+Normbm[]/2}];


(********* Define time vector and Lie rule to get time derivative *********)
BmToTimeRule := MakeRule[{LieD[n[b]][Bm[-a]], 1/Lapsen[] LieD[t[b]- \[Beta][b]][Bm[-a]]}];